unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, RackCtls, ueled, uEGauge, uebutton, uEMultiTurn,
  uEKnob, uERotImage, uETilePanel, SdpoSerial, Forms, Controls, Graphics,
  Dialogs, strutils, StdCtrls, ExtCtrls, ComCtrls;

type

    TMeterRangeData = record
      formatString : string;
      unitString : string;
      scale : integer;
      rangeString : string;
    end;

  { TMainForm }

  TMainForm = class(TForm)
    LabelDisplay: TLabel;
    LabelMode: TLabel;
    LabelRange: TLabel;
    LabelOver: TLabel;
    ListBoxCommands: TListBox;
    Panel1: TPanel;
    ScrewPanel1: TScrewPanel;
    SdpoSerial: TSdpoSerial;
    TimerRead: TTimer;
    TimerSendCommand: TTimer;
    uELEDRead: TuELED;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure SdpoSerialRxData(Sender: TObject);
    procedure TimerReadTimer(Sender: TObject);
    procedure TimerSendCommandTimer(Sender: TObject);
  private
    { private declarations }
    rxBuffer : string;
    cmdType : boolean;
    procedure ResetLabels;
    procedure ProcessV543Data ( rawHex : string );
    procedure UpdateDisplay ( reading : integer; modeId : integer; rangeId :integer; polarity : integer; over : boolean );
  public
    { public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

{ TMainForm }

procedure TMainForm.ResetLabels;
begin
  self.LabelDisplay.Caption := '0';
  self.LabelMode.Caption  := '-';
  self.LabelRange.Caption  := '-';
  self.LabelOver.Caption  := '-';
end;

procedure TMainForm.ProcessV543Data ( rawHex : string );
var raw : LongInt;
    reading : integer;
    rangeId : integer;
    modeId : integer;
    polarity : integer;
    over : boolean;
begin
     self.caption := 'Meratronik V543 Simple Panel (' + rawHex + ')';
     raw := Hex2Dec( rawHex );
     reading := StrToInt( HexStr( (raw and $0001FFFF), 5 ) );
     rangeId := (raw shr 17) and $07;
     modeId := (raw shr 22) and $07;
     polarity := (raw shr 20) and $03;
     over := not odd(raw shr 25);
     UpdateDisplay ( reading, modeId, rangeId, polarity, over );
end;


const voltage : array[0..7] of TMeterRangeData = (
      (* 00 *) (formatString : '#00.00'; unitString : ' V'; scale : 100;   rangeString : '100V' ),
      (* 01 *) (formatString : '#.0000'; unitString : ' V'; scale : 10000; rangeString : '1V'   ),
      (* 02 *) (formatString : '#000.0'; unitString : ' V'; scale : 10;    rangeString : '1000V'),
      (* 03 *) (formatString : '#0.000'; unitString : ' V'; scale : 1000;  rangeString : '10V'  ),
      (* 04 *) (formatString : '#00.00'; unitString : 'mV'; scale : 100;   rangeString : '100mV'),
      (* 05 *) (formatString : '#00.00'; unitString : '--'; scale : 1;     rangeString : '---'  ),
      (* 06 *) (formatString : '#00.00'; unitString : '--'; scale : 1;     rangeString : '---'  ),
      (* 07 *) (formatString : '#00.00'; unitString : '--'; scale : 1;     rangeString : '---'  )
);

const resistance : array[0..7] of TMeterRangeData = (
      (* 00 *) (formatString : '#00.00'; unitString : 'kΩ'; scale : 100;    rangeString : '100kΩ' ),
      (* 01 *) (formatString : '#.0000'; unitString : 'kΩ'; scale : 10000;  rangeString : '1kΩ'   ),
      (* 02 *) (formatString : '#000.0'; unitString : '--'; scale : 1;      rangeString : '---'),
      (* 03 *) (formatString : '#0.000'; unitString : 'kΩ'; scale : 1000;   rangeString : '10kΩ'  ),
      (* 04 *) (formatString : '#00.00'; unitString : '--'; scale : 1;      rangeString : '---'),
      (* 05 *) (formatString : '#.0000'; unitString : 'MΩ'; scale : 10000;  rangeString : '1MΩ'  ),
      (* 06 *) (formatString : '#00.00'; unitString : '--'; scale : 1;      rangeString : '---'  ),
      (* 07 *) (formatString : '#0.000'; unitString : 'MΩ'; scale : 1000;   rangeString : '10MΩ'  )
);


procedure TMainForm.UpdateDisplay ( reading : integer; modeId : integer; rangeId :integer; polarity : integer; over : boolean );
var pol : string;
begin
     pol := ' ';
     case modeId of
          $01:
             begin
               self.LabelMode.Caption := 'R';
               self.LabelRange.Caption := resistance[ rangeId ].rangeString;
               pol := ' ';
               self.LabelDisplay.Caption := pol + FormatFloat ( resistance[ rangeId ].formatString, reading / resistance[ rangeId ].scale ) + resistance[ rangeId ].unitString;
             end;
          $02:
             begin
               self.LabelMode.Caption := 'AC';
               self.LabelRange.Caption := voltage[ rangeId ].rangeString;
               pol := '~';
               self.LabelDisplay.Caption := pol + FormatFloat ( voltage[ rangeId ].formatString, reading / voltage[ rangeId ].scale ) + voltage[ rangeId ].unitString;
             end;
          $04:
             begin
               self.LabelMode.Caption := 'DC';
               self.LabelRange.Caption := voltage[ rangeId ].rangeString;
               if polarity = $01 then pol := '+' else pol := '-';
               self.LabelDisplay.Caption := pol + FormatFloat ( voltage[ rangeId ].formatString, reading / voltage[ rangeId ].scale ) + voltage[ rangeId ].unitString;
             end
     else
       self.LabelMode.Caption := 'err';
     end;

     if over then
        self.LabelOver.Caption := 'OVER'
     else
       self.LabelOver.Caption := '';

end;


procedure TMainForm.FormCreate(Sender: TObject);
begin
  ResetLabels;
  self.rxBuffer := '';
  self.SdpoSerial.Device := '/dev/ttyACM0';
  self.SdpoSerial.BaudRate := br__9600;
  self.SdpoSerial.Open;
end;

procedure TMainForm.SdpoSerialRxData(Sender: TObject);
begin
   self.rxBuffer := self.rxBuffer + self.SdpoSerial.ReadData;
   if RightStr( self.rxBuffer, 1 ) = chr( 10 ) then
   begin
     ProcessV543Data ( Trim( self.rxBuffer ) );
     self.rxBuffer := '';
   end;
end;

procedure TMainForm.TimerReadTimer(Sender: TObject);
begin
  self.uELEDRead.Active := true;
  self.cmdType := not self.cmdType;
  if self.cmdType then
     self.ListBoxCommands.Items.Add ( 'START' )
  else
     self.ListBoxCommands.Items.Add ( 'READ' );
end;

procedure TMainForm.TimerSendCommandTimer(Sender: TObject);
begin
   if self.ListBoxCommands.Count > 0 then
   begin
      self.uELEDRead.Active := false;
      SdpoSerial.WriteData( self.ListBoxCommands.Items[ 0 ] + chr(10) );
      self.ListBoxCommands.Items.Delete( 0 );
   end;
end;

procedure TMainForm.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
     self.SdpoSerial.Close;
end;


end.

